<?php
if (!defined('ADMIN_MOD_INSTALL')) { exit; }

if( !defined('ROSTER_DF_INSTALLER') )
{
	define('ROSTER_DF_INSTALLER',true);
}

# module installer
function RosterDF_install($this_prefix, $this_base)
{
	global $installer, $db, $prefix, $BASEHREF;

	if( version_compare(CPG_NUKE, '9.1.1', '<') )
	{
		cpg_error('WoWRosterDF requires DragonFly version 9.1.1');
	}

	define('ROSTER_DB_DIR',  ROSTER_LIB . 'dbal' . DIR_SEP);

	$db_structure_file = ROSTER_DB_DIR . 'structure' . DIR_SEP . 'mysql_structure.sql';
	$db_data_file      = ROSTER_DB_DIR . 'structure' . DIR_SEP . 'mysql_data.sql';

	// Parse structure file and create database tables
	$sql = @fread(@fopen($db_structure_file, 'r'), @filesize($db_structure_file));
	$sql = preg_replace('#renprefix\_(\S+?)([\s\.,]|$)#', $prefix . '_' . $this_prefix . '\\1\\2', $sql);

	$sql = RosterDF_remove_remarks($sql);
	$sql = RosterDF_parse_sql($sql, ';');

	$sql_count = count($sql);
	$i = 0;

	while( $i < $sql_count )
	{
		if( isset($sql[$i]) && $sql[$i] != '' )
		{
			$db->sql_query($sql[$i]);
		}
		$i++;
	}
	unset($sql);

	// Parse the data file and populate the database tables
	$sql = @fread(@fopen($db_data_file, 'r'), @filesize($db_data_file));
	$sql = preg_replace('#renprefix\_(\S+?)([\s\.,]|$)#', $prefix . '_' . $this_prefix . '\\1\\2', $sql);

	$sql = RosterDF_remove_remarks($sql);
	$sql = RosterDF_parse_sql($sql, ';');

	$sql_count = count($sql);
	$i = 0;

	while( $i < $sql_count )
	{
		if( isset($sql[$i]) && $sql[$i] != '' )
		{
			$db->sql_query($sql[$i]);
		}
		$i++;
	}

	unset($sql);

	/**
	 * Update some config settings
	 */
	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = 'enUS' WHERE `config_name` = 'locale';");
	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = '" . ROSTER_VERSION . "' WHERE `config_name` = 'version';");
	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = '$BASEHREF' WHERE `config_name` = 'website_address';");

	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = 'rosterdf' WHERE `config_name` = 'theme';");

	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `form_type` = 'display' WHERE `config_name` = 'seo_url';");

	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = 'modules/" . $this_base . "/img/' WHERE `config_name` = 'img_url';");
	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = 'modules/" . $this_base . "/img/' WHERE `config_name` = 'interface_url';");

	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = 'modules/" . $this_base . "/img/wowroster_logo.jpg' WHERE `config_name` = 'logo';");
	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "config` SET `config_value` = 'modules/" . $this_base . "/img/wowroster_bg.jpg' WHERE `config_name` = 'roster_bg';");

	$db->sql_query("INSERT INTO `" . $prefix . '_' . $this_prefix . "account` (`account_id`, `name`) VALUES (1, 'Guild'), (2, 'Officer'), (3, 'Admin');");
	$db->sql_query("UPDATE `" . $prefix . '_' . $this_prefix . "account` SET `hash` = '" . md5('admin') . "';");

	return true;
}

# module uninstaller
function RosterDF_uninstall($this_prefix)
{
	global $installer, $prefix, $db;

	$db->sql_query("SHOW TABLES LIKE '" . $prefix . '_' . $this_prefix . "%';");

	while( $row = $db->sql_fetchrow(0,SQL_NUM) )
	{
		$table = str_replace($prefix . '_','',$row[0]);
		$installer->add_query('DROP', $table);
		print $table . '<br />';
	}
	return true;
}

# module upgrader
function RosterDF_upgrade($prev_version, $this_prefix, $this_base)
{
	global $installer, $db, $prefix;

	if( version_compare(CPG_NUKE, '9.1.1', '<') )
	{
		cpg_error('WoWRosterDF requires DragonFly version 9.1.1');
	}

	if( version_compare($prev_version, '1.9.9', '<') )
	{
		cpg_error('WoWRosterDF 2.0 Cannot Upgrade 1.7.x');
	}

	return true;
}


/**
 * Removes comments from a SQL data file
 *
 * @param    string  $sql    SQL file contents
 * @return   string
 */
function RosterDF_remove_remarks($sql)
{
	if( $sql == '' )
	{
		die('Could not obtain SQL structure/data');
	}

	$retval = '';
	$lines  = explode("\n", $sql);
	unset($sql);

	foreach( $lines as $line )
	{
		// Only parse this line if there's something on it, and we're not on the last line
		if ( strlen($line) > 0 )
		{
			// If '#' is the first character, strip the line
			$retval .= ( substr($line, 0, 1) != '#' ) ? $line . "\n" : "\n";
		}
	}
	unset($lines, $line);

	return $retval;
}

/**
 * Parse multi-line SQL statements into a single line
 *
 * @param    string  $sql    SQL file contents
 * @param    char    $delim  End-of-statement SQL delimiter
 * @return   array
 */
function RosterDF_parse_sql( $sql , $delim )
{
	if ( $sql == '' )
	{
		die('Could not obtain SQL structure/data');
	}

	$retval     = array();
	$statements = explode($delim, $sql);
	unset($sql);

	$linecount = count($statements);
	for( $i = 0; $i < $linecount; $i++ )
	{
		if( ($i != $linecount - 1) || (strlen($statements[$i]) > 0) )
		{
			$statements[$i] = trim($statements[$i]);
			$statements[$i] = str_replace("\r\n", '', $statements[$i]) . "\n";

			// Remove 2 or more spaces
			$statements[$i] = preg_replace('#\s{2,}#', ' ', $statements[$i]);

			$retval[] = trim($statements[$i]);
		}
	}
	unset($statements);

	return $retval;
}
